from terraformmetrics.terraform_metric import TerraformMetric


class NumModules(TerraformMetric):
    """ This class measures the number of modules used
    """

    def count(self):
        """Return the number of modules used
        Returns
        -------
        int
            Number of modules used
        """

        num_modules = 0

        if 'module' in self.tfparsed:
            num_modules = len(self.tfparsed['module'])

        return num_modules



from terraformmetrics.terraform_metric import TerraformMetric


class NumDependsOn(TerraformMetric):
    """ This class measures the number of occurrences of the depends_on parameter
    """

    def count(self):
        """Return the number of occurrences of the depends_on parameter
        Returns
        -------
        int
             number of occurrences of the depemds_on parameter
        """
        count = 0

        def count_depends_on(data):
            counter = 0
            if isinstance(data, dict):
                counter = sum(1 for key, value in data.items() if key == "depends_on")
                for value in data.values():
                    counter += count_depends_on(value)
            elif isinstance(data, list):
                counter = sum(count_depends_on(item) for item in data)
            return counter

        num_depends = count_depends_on(self.tfparsed)
        return num_depends



import re
from terraformmetrics.terraform_metric import TerraformMetric

PATH_REGEX = re.compile(r'\b[\w/.-]+(?:/+[\w/.-]*)+\b')


class NumPath(TerraformMetric):
    """ This class measures the number of paths in a Script Terraform
    """

    def count(self):
        """Return the number of paths
        Returns
        -------
        int
            Number of paths
        """

        # Funzione ricorsiva per contare il numero di path all'interno di un file
        def count_paths(data):
            count = 0
            if isinstance(data, dict):
                for key, value in data.items():
                    count += count_paths(value)
            elif isinstance(data, list):
                for item in data:
                    count += count_paths(item)
            elif isinstance(data, str):
                path_matches = PATH_REGEX.findall(data)
                ip_matches = re.findall(r'\b(?:\d{1,3}\.){3}\d{1,3}/\d{1,2}\b', data)
                filtered_matches = [match for match in path_matches if match not in ip_matches]
                count += len(filtered_matches)
            return count

        total_paths_count = count_paths(self.tfparsed)

        return total_paths_count
    


from terraformmetrics.terraform_metric import TerraformMetric


class NumFilePermissions(TerraformMetric):
    """ This class measures the number of occurrences of the file_permission parameter within the "file"
    provisioner block to specify the permissions of the files being copied or created
    """

    def count(self):
        """Return the number of occurrences of the file_permission parameter
        Returns
        -------
        int
             number of occurrences of the file_permission parameter
        """
        count = 0

        def count_file_permission(data):
            counter = 0

            if isinstance(data, dict):
                for key, value in data.items():
                    if key == 'provisioner' and isinstance(value, list):
                        for provisioner in value:
                            if isinstance(provisioner, dict):
                                for sub_key, sub_value in provisioner.items():
                                    if sub_key == 'file':
                                        if 'file_permission' in provisioner['file']:
                                            counter += 1
                    else:
                        counter += count_file_permission(value)

            elif isinstance(data, list):
                for item in data:
                    counter += count_file_permission(item)

            return counter

        num_file_permission = count_file_permission(self.tfparsed)
        return num_file_permission



from terraformmetrics.terraform_metric import TerraformMetric

import re


class NumErrorHandling(TerraformMetric):
    """ The try function is used for error handling, the method counts the number of occurrences of the latter
    """

    def count(self):
        """Return the number of try functions
        Returns
        -------
        int
            Number of try functions
        """

        # Funzione ricorsiva per contare il numero di funzioni try
        def count_try(data):
            count = 0
            if isinstance(data, dict):
                for key, value in data.items():
                    count += count_try(value)
            elif isinstance(data, list):
                for item in data:
                    count += count_try(item)
            elif isinstance(data, str):
                if re.search(r'try\(', data):
                    count += 1
            return count

        total_try_count = count_try(self.tfparsed)

        return total_try_count



from terraformmetrics.terraform_metric import TerraformMetric

class NumResources(TerraformMetric):
    """ This class measures the number of reources in a Script Terraform

    resources are identified by counting the number of occurrences of the keyword resource
    """

    def count(self):
        """Return the number of resources
        Returns
        -------
        int
            Number of resources
        """
        resource_list = self.tfparsed.get('resource', [])
        resource = len(resource_list)

        return resource






from terraformmetrics.import_metrics import general_metrics, tf_metrics

def extract_all(script: str):
    if script is None:
        raise TypeError('Expected a string')

    metrics = general_metrics
    metrics.update(tf_metrics)

    results = dict()

    # Execute metrics
    for name in metrics:
        results[name] = metrics[name](script).count()

    return results



from terraformmetrics.terraform_metric import TerraformMetric


class NumProviders(TerraformMetric):
    """ This class measures the number of providers used
    """

    def count(self):
        """Return the number of providers used
        Returns
        -------
        int
            Number of providers used
        """

        num_providers = 0

        if 'provider' in self.tfparsed:
            num_providers = len(self.tfparsed['provider'])

        return num_providers



from terraformmetrics.terraform_metric import TerraformMetric


class NumLocals(TerraformMetric):
    """ This class measures the number of local variables
    """

    def count(self):
        """Return the number of locals
        Returns
        -------
        int
            Number of locals
        """

        # Funzione ricorsiva per contare il numero di variabili locali
        def count_local_variables(data):
            count = 0

            if isinstance(data, dict):
                for key, value in data.items():
                    if key == "locals":
                        count += sum(1 for _ in value)
                    else:
                        count += count_local_variables(value)
            elif isinstance(data, list):
                for item in data:
                    count += count_local_variables(item)

            return count

        total_locals_count = count_local_variables(self.tfparsed)

        return total_locals_count



from terraformmetrics.terraform_metric import TerraformMetric


class NumHttp(TerraformMetric):
    """ This class measures the number of http data source used
    """

    def count(self):
        """Return the number of http data source used
        Returns
        -------
        int
            Number of http data source used
        """

        num_http = 0

        if 'data' in self.tfparsed:
            for elem in self.tfparsed['data']:
                if 'http' in elem:
                    num_http += 1

        return num_http



from terraformmetrics.lines_metric import LinesMetric


class LinesBlank(LinesMetric):
    """ This class measures the blank lines in a Terraform script"""

    def count(self):
        """Return the number of blank lines

        -------
        int
            Number of blank lines
        """
        bloc = 0

        for line in self.hcl.splitlines():
            if not line.strip():
                bloc += 1

        return bloc


from terraformmetrics.terraform_metric import TerraformMetric


class NumRemoteExec(TerraformMetric):
    """ This class measures the number of "remote-exec" provisioner
    """

    def count(self):
        """Return the number of "remote-exec" provisioner
        Returns
        -------
        int
             number of "remote-exec" provisioner
        """
        count = 0

        def count_remote_exec_provisioners(data):
            count = 0

            if isinstance(data, dict):
                for key, value in data.items():
                    if key == 'provisioner' and isinstance(value, list):
                        for provisioner in value:
                            if isinstance(provisioner, dict):
                                for sub_key, sub_value in provisioner.items():
                                    if sub_key == 'remote-exec':
                                        count += 1
                    else:
                        count += count_remote_exec_provisioners(value)

            elif isinstance(data, list):
                for item in data:
                    count += count_remote_exec_provisioners(item)

            return count

        num_remote = count_remote_exec_provisioners(self.tfparsed)
        return num_remote



from terraformmetrics.lines_metric import LinesMetric


class NumTokens(LinesMetric):
    """ This class measures the numer of tokens, i.e. the number of words separated by a blank space"""

    def count(self):
        """Return the number of words separated by a blank lines

        -------
        int
            Number of words separated by a blank lines
        """
        tokens = self.hcl.split()

        # Filtra i token significativi (parole chiave, nomi di risorse, attributi)
        significant_tokens = [token for token in tokens if token.strip()]

        # Conta il numero di token significativi
        num_significant_tokens = len(significant_tokens)

        return num_significant_tokens